import xbmc
from helper import pluginmenu, utils
from . import common, genre, tag, studio, person


class Videos:
    def __init__(self, EmbyServer, SQLs):
        self.EmbyServer = EmbyServer
        self.SQLs = SQLs
        self.GenreObject = genre.Genre(EmbyServer, self.SQLs)
        self.TagObject = tag.Tag(EmbyServer, self.SQLs)
        self.StudioObject = studio.Studio(EmbyServer, self.SQLs)
        self.PersonObject = person.Person(EmbyServer, self.SQLs)

    def change(self, Item):
        common.load_ExistingItem(Item, self.EmbyServer, self.SQLs["emby"], "Video")

        if not common.verify_content(Item, "video"):
            return False

        if not common.get_file_path(Item, "movies", Item['LibraryId'], self.EmbyServer.ServerData['ServerId']):
            return False

        xbmc.log(f"EMBY.core.videos: Process Item: {Item['Name']}", 0) # DEBUG
        common.SwopMediaSources(Item)  # 3D
        common.set_trailer(Item, self.EmbyServer)
        common.set_RunTimeTicks(Item)
        common.get_streams(Item)
        common.set_common(Item, self.EmbyServer.ServerData['ServerId'], False)
        common.set_chapters(Item, self.EmbyServer.ServerData['ServerId'])

        if self.SQLs["video"]: # Sync content to Kodi db
            Item['TagItems'].append({"LibraryId": Item["LibraryId"], "Type": "Tag", "Id": f"999999993{Item['LibraryId']}", "Name":  f"{Item['LibraryName']} (Library)", "Memo": "library"})
            common.set_MetaItems(Item, self.SQLs, self.GenreObject, self.EmbyServer, "Genre", "GenreItems")
            common.set_MetaItems(Item, self.SQLs, self.StudioObject, self.EmbyServer, "Studio", "Studios")
            common.set_MetaItems(Item, self.SQLs, self.TagObject, self.EmbyServer, "Tag", 'TagItems')
            common.set_people(Item, self.SQLs, self.PersonObject, self.EmbyServer)
            self.SQLs["emby"].add_streamdata(Item['Id'], Item['Streams'])

            if Item['UpdateItem']:
                common.delete_ContentItemReferences(Item, self.SQLs, "movie")
            else:
                Item['KodiPathId'] = self.SQLs['video'].get_add_path(Item['Path'], "movies")
                Item['KodiItemId'] = self.SQLs["video"].create_movie_entry()
                Item['KodiFileId'] = self.SQLs["video"].create_entry_file()

            common.set_ContentItem(Item, self.SQLs, self.EmbyServer, "movie")
            common.set_Genre_links(Item['KodiItemId'], self.SQLs, "movie", Item["GenreItems"])
            common.set_Studio_links(Item['KodiItemId'], self.SQLs, "movie", Item["Studios"])
            common.set_Tag_links(Item['KodiItemId'], self.SQLs, "movie", Item["TagItems"])
            common.set_Actor_links(Item['KodiItemId'], self.SQLs, "movie", Item["CastItems"])
            common.set_Writer_links(Item['KodiItemId'], self.SQLs, "movie", Item["WritersItems"])
            common.set_Director_links(Item['KodiItemId'], self.SQLs, "movie", Item["DirectorsItems"])
            self.SQLs["video"].set_Favorite_Tag(Item['UserData']['IsFavorite'], Item['KodiItemId'], "movie")
            Item['Unique'] = self.SQLs["video"].add_uniqueids(Item['KodiItemId'], Item['ProviderIds'], "movie", 'imdb')
            Item['RatingId'] = self.SQLs["video"].add_ratings(Item['KodiItemId'], "movie", "default", Item['CommunityRating'])

            if not Item['ProductionLocations']:
                Item['ProductionLocations'].append("")

            if Item['CriticRating']:
                Item['CriticRating'] = float(Item['CriticRating'] / 10.0)
                self.SQLs["video"].add_ratings(Item['KodiItemId'], "movie", "tomatometerallcritics", Item['CriticRating'])

            if Item['UpdateItem']: # new item
                self.SQLs["video"].update_movie(Item['KodiItemId'], Item['KodiFileId'], Item['Name'], Item['Overview'], Item['ShortOverview'], Item['Tagline'], Item['RatingId'], Item['Writers'], Item['KodiArtwork']['poster'], Item['Unique'], Item['SortName'], Item['RunTimeTicks'], Item['OfficialRating'], Item['Genre'], Item['Directors'], Item['OriginalTitle'], Item['Studio'], Item['Trailer'], Item['KodiArtwork']['fanart'].get('fanart', ""), Item['ProductionLocations'][0], Item['PremiereDate'], Item['UserData']['PlayCount'], Item['UserData']['LastPlayedDate'], None, Item['Filename'])
                self.SQLs["emby"].update_reference_video(Item['Id'], Item['UserData']['IsFavorite'], Item['ParentId'], Item['PresentationUniqueKey'])
                xbmc.log(f"EMBY.core.videos: UPDATE {Item['Id']}: {Item['Name']}", 1) # LOGINFO
            else:
                self.SQLs["video"].add_movie(Item['KodiItemId'], Item['KodiFileId'], Item['Name'], Item['Overview'], Item['ShortOverview'], Item['Tagline'], Item['RatingId'], Item['Writers'], Item['KodiArtwork']['poster'], Item['Unique'], Item['SortName'], Item['RunTimeTicks'], Item['OfficialRating'], Item['Genre'], Item['Directors'], Item['OriginalTitle'], Item['Studio'], Item['Trailer'], Item['KodiArtwork']['fanart'].get('fanart', ""), Item['ProductionLocations'][0], Item['Path'], Item['KodiPathId'], Item['PremiereDate'], Item['Filename'], Item['DateCreated'], Item['UserData']['PlayCount'], Item['UserData']['LastPlayedDate'], None)
                self.SQLs["emby"].add_reference_video(Item['Id'], Item['LibraryId'], Item['KodiItemId'], Item['UserData']['IsFavorite'], Item['KodiFileId'], Item['ParentId'], Item['PresentationUniqueKey'], Item['EmbyPath'], Item['KodiPathId'])

            utils.FavoriteQueue.put(((Item['KodiArtwork']['thumb'], Item['UserData']['IsFavorite'], f"{Item['Path']}{Item['Filename']}", Item['Name'], "media", 0),))
        else: # Unsync content to Kodi db (e.g. Emby specials)
            common.get_filename(Item, self.EmbyServer.API, "movie")
            common.SwopMediaSources(Item)  # 3D
            self.SQLs["emby"].add_streamdata(Item['Id'], Item['Streams'])
            self.SQLs["emby"].add_reference_video(Item['Id'], Item['LibraryId'], None, Item['UserData']['IsFavorite'], None, Item['ParentId'], Item['PresentationUniqueKey'], Item['EmbyPath'], None)
            xbmc.log(f"EMBY.core.videos: ADD {Item['Id']}: {Item['Name']}", 1) # LOGINFO

        self.SQLs["emby"].add_multiversion(Item, "Video", self.EmbyServer.API, self.SQLs)
        return not Item['UpdateItem']

    # This updates: Favorite, LastPlayedDate, Playcount, PlaybackPositionTicks
    def userdata(self, Item):
        if Item['PlayedPercentage'] and Item['PlayedPercentage']:
            RuntimeSeconds = int(Item['PlaybackPositionTicks'] / Item['PlayedPercentage'] / 100000)
        else:
            RuntimeSeconds = 0

        common.set_playstate(Item)
        self.SQLs["video"].set_Favorite_Tag(Item['IsFavorite'], Item['KodiItemId'], "movie")
        self.SQLs["video"].update_bookmark_playstate(Item['KodiFileId'], Item['PlayCount'], Item['LastPlayedDate'], Item['PlaybackPositionTicks'], RuntimeSeconds)
        self.SQLs["emby"].update_favourite(Item['IsFavorite'], Item['Id'], "Video")
        self.set_favorite(Item['IsFavorite'], Item['KodiFileId'], Item['KodiItemId'])
        pluginmenu.reset_querycache("Video")
        xbmc.log(f"EMBY.core.videos: New resume point {Item['Id']}: {Item['PlaybackPositionTicks']}", 0) # LOGDEBUG
        xbmc.log(f"EMBY.core.videos: USERDATA [{Item['KodiFileId']} / {Item['KodiItemId']}] {Item['Id']}", 1) # LOGINFO

    def remove(self, Item):
        if common.delete_ContentItem(Item, self.SQLs, "movie", "Video"):
            self.SQLs["video"].delete_movie(Item['KodiItemId'], Item['KodiFileId'])
            xbmc.log(f"EMBY.core.video: DELETE [{Item['KodiItemId']} / {Item['KodiFileId']}] {Item['Id']}", 1) # LOGINFO
            self.set_favorite(False, Item['KodiFileId'], Item['KodiItemId'])

            if not Item['LibraryId']:
                StackedIds = self.SQLs["emby"].get_EmbyIds_by_EmbyPresentationKey(Item['PresentationUniqueKey'], "Video")

                if StackedIds: # multi version force update
                    xbmc.log(f"EMBY.core.movies: DELETE multi version movies from embydb {Item['Id']}", 1) # LOGINFO

                    for StackedId in StackedIds:
                        self.SQLs["emby"].add_RemoveItem(StackedId, None)
                        self.SQLs["emby"].add_UpdateItem(StackedId, "Video", "unknown")
        else:
            self.SQLs["video"].delete_links_tags(Item['KodiItemId'], "movie")

    def set_favorite(self, IsFavorite, KodiFileId, KodiItemId):
        FullPath, Thumbnail, Itemname = self.SQLs["video"].get_favoriteData(KodiFileId, KodiItemId, "movie")
        utils.FavoriteQueue.put(((Thumbnail, IsFavorite, FullPath, Itemname, "media", 0),))
